#ifndef STARSIDAProgram_h
#define STARSIDAProgram_h

#include <list>
// #include <map>
// #include <set>
// #include <string>

// #include <cstdint>
// #include <cstdio>

// #include <intel.hpp>  // indirectly includes the unguarded allins.hpp header

// #include "interfaces/STARSTypes.h"
#include "interfaces/abstract/STARSProgram.h"

#include "base/SMPProgram.h"

class STARS_IDA_Program_t : public STARS_Program_t {
public:
	// Data initialization
	virtual void InitData(void);
	void DetermineRootFileName(void);
	bool OpenFiles(void);
	void CloseFiles(void);
	virtual bool OpenMainAnnotationFile(void);
	virtual bool OpenSecondaryFilesOnly(void);

	// Get (accessor) methods

	// Set (mutator) methods

	// Query methods
	virtual bool IsCodeAddressTaken(const STARS_ea_t InstAddr) const { 
		return (CodeAddressesTaken.find(InstAddr) != CodeAddressesTaken.cend());
	};

	// Printing methods
	virtual void PrintAllAddressTakenXrefs(STARS_ea_t InstAddr, std::size_t InstSize) override;
	virtual void PrintAllCodeToCodeXrefs(STARS_ea_t InstAddr, std::size_t InstSize, bool CallFlag, bool CompleteIndirCall);

	// Analysis methods

	// NOTE: Change this when we do targets with data (other than switch tables) in code.
	//  Call IsDataAddress() first to weed out data items in the code range.
	bool IsImmedCodeAddress(STARS_ea_t ImmedValue) {
		STARS_Segment_t *seg = global_stars_interface->getseg(ImmedValue);
		bool FoundCodeAddress = false;
		if (nullptr != seg) {
			FoundCodeAddress = seg->IsCodeSegment();
		}
		return FoundCodeAddress;
	}

	bool IsImmedDataAddress(STARS_ea_t ImmedValue) {
		STARS_Segment_t *seg = global_stars_interface->getseg(ImmedValue);
		bool FoundDataAddress = false;
		if (nullptr != seg) {
			FoundDataAddress = seg->IsDataSegment();
		}
		return FoundDataAddress;
	}

	void ReportTotalCodeSize(unsigned long long TotalCodeSize);
	void InitStaticDataTable(SMPProgram *CurrProg);
	void GetBlockSuccessorTargets(bool CallFlag, STARS_InstructionID_t LastBlockInst, std::size_t InstSize, std::list<STARS_InstructionID_t> &SuccList); // Get successors for LastBlockInst, omitting call targets
	virtual void FindCodeAddressesTaken(SMPProgram *CurrProg); // find code addresses in data

private:
	// Data
	std::size_t STARS_Platform_Bitwidth; // not binary, but the system OS we are running on
	std::set<STARS_ea_t> CodeAddressesTaken; // instruction addresses found in data segments

	// Methods
	void ComputeGlobalFieldOffsets(struct GlobalVar &CurrGlobal);
	
};

#endif
