#ifndef STARSIDASegment_h
#define STARSIDASegment_h

#include <pro.h>
#include <segment.hpp>
#include <bytes.hpp>

#include "interfaces/STARSTypes.h"

class STARS_IDA_Segment_t : public STARS_Segment_t
{
	public:
		STARS_IDA_Segment_t(segment_t* seg) : the_seg(seg) {}

#if (IDA_SDK_VERSION < 700)
		virtual STARS_ea_t get_startEA() { return the_seg->startEA; }
		virtual STARS_ea_t get_endEA() { return the_seg->endEA; } 
#else
		virtual STARS_ea_t get_startEA() { return the_seg->start_ea; }
		virtual STARS_ea_t get_endEA() { return the_seg->end_ea; }
#endif
		virtual std::size_t get_seg_size() { return (std::size_t)(get_endEA() - get_startEA()); }
		virtual bool IsCodeSegment() const { return the_seg->type == SEG_CODE; }
		virtual bool IsXternSegment() const { return the_seg->type == SEG_XTRN; }
		virtual bool IsDataSegment() const { return the_seg->type == SEG_DATA; }
		virtual bool IsCommonSegment() const { return the_seg->type == SEG_COMM; }
		virtual bool IsBSSSegment() const { return the_seg->type == SEG_BSS; }
		virtual bool IsReadableSegment() const { return the_seg->perm & SEGPERM_READ; }
		virtual bool IsWriteableSegment() const { return the_seg->perm & SEGPERM_WRITE; }
#if (IDA_SDK_VERSION < 700)
		virtual STARS_ssize_t GetSegmentName(char* name, int len) { return ::get_segm_name(the_seg, name, len); }
#else
		virtual STARS_ssize_t GetSegmentName(char* name, int len) {
			qstring *temp = new qstring();
			temp->resize(len);
			STARS_ssize_t SegSize = ::get_visible_segm_name(temp, the_seg); 
			qstrncpy(name, temp->c_str(), len);
			delete temp;
			return SegSize;
		}
#endif

		// Fetch read-only data values, if any.
		virtual bool GetReadOnlyMem64BitValue(STARS_ea_t MemAddr, uint64_t &Value) const {
//			assert(!IsWriteableSegment());
			Value = (uint64_t) ::get_qword(MemAddr);
			return true;
		}
		virtual bool GetReadOnlyMem32BitValue(STARS_ea_t MemAddr, uint32_t &Value) const {
//			assert(!IsWriteableSegment());
#if (IDA_SDK_VERSION < 700)
			Value = (uint32_t) ::get_long(MemAddr);
#else
			Value = (uint32_t) ::get_dword(MemAddr);
#endif
			return true;
		}
		virtual bool GetReadOnlyMem16BitValue(STARS_ea_t MemAddr, uint16_t &Value) const {
//			assert(!IsWriteableSegment());
			Value = (uint16_t) ::get_word(MemAddr);
			return true;
		}
		virtual bool GetReadOnlyMem8BitValue(STARS_ea_t MemAddr, uint8_t &Value) const {
//			assert(!IsWriteableSegment());
			Value = (uint8_t) ::get_byte(MemAddr);
			return true;
		}

	private:
		segment_t* the_seg;
		
};

#endif
