
#include <assert.h>
#include <interfaces/irdb/STARSSegment.h>

template<class T>
bool STARS_IRDB_Segment_t::GetReadOnlyMemValue(STARS_ea_t MemAddr, T &Value) const 
{
	// can't do this for writeable segments.
	if(IsWriteableSegment())
		return false;

	// possible to have read-only bss sections.
	if(IsBSSSegment())
	{
		Value=0; 
		return true;
	}

	// check for in-the-range of this segment.
	if (
		// start < mem
		the_seg->get_address() <= MemAddr && 
		// and mem+size < end
		(MemAddr+sizeof(T)-1) < (the_seg->get_address()+the_seg->get_size())
	   )
	{	
	
		// valid range in a read-only section. 

		// get data.
		const char* data=the_seg->get_data();
		Value=0;
		// build up the integer of the size requested.
		for(auto i=0U;i<sizeof(T);i++)
		{
			Value|= ((T)data[MemAddr-the_seg->get_address()+i]) << (8*i);
		}

		// return that we succeeded.
		return true;
	}
	else
		return false;
}

bool STARS_IRDB_Segment_t::GetReadOnlyMem64BitValue(STARS_ea_t MemAddr, uint64_t &Value) const 
{ return GetReadOnlyMemValue<uint64_t>(MemAddr,Value); }

bool STARS_IRDB_Segment_t::GetReadOnlyMem32BitValue(STARS_ea_t MemAddr, uint32_t &Value) const 
{ return GetReadOnlyMemValue<uint32_t>(MemAddr,Value); }

bool STARS_IRDB_Segment_t::GetReadOnlyMem16BitValue(STARS_ea_t MemAddr, uint16_t &Value) const 
{ return GetReadOnlyMemValue<uint16_t>(MemAddr,Value); }

bool STARS_IRDB_Segment_t::GetReadOnlyMem8BitValue(STARS_ea_t MemAddr, uint8_t &Value) const 
{ return GetReadOnlyMemValue<uint8_t>(MemAddr,Value); }

